#!/usr/bin/env python
# encoding: utf-8

#*    dsgui - a graphical user interface for Datove schranky
#*    Copyright (C) 2009-2010  CZ.NIC, z.s.p.o. (http://www.nic.cz)
#*
#*    This library is free software; you can redistribute it and/or
#*    modify it under the terms of the GNU Library General Public
#*    License as published by the Free Software Foundation; either
#*    version 2 of the License, or (at your option) any later version.
#*
#*    This library is distributed in the hope that it will be useful,
#*    but WITHOUT ANY WARRANTY; without even the implied warranty of
#*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#*    Library General Public License for more details.
#*
#*    You should have received a copy of the GNU Library General Public
#*    License along with this library; if not, write to the Free
#*    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import time
START_TIME = time.time()

import os, sys
topdir = os.path.abspath(os.path.dirname(__file__))
  
import locale
# gettext installation
if os.name == 'nt':
 # windows hack for locale setting
 lang = os.getenv('LANG')
 if lang is None:
  default_lang, default_enc = locale.getdefaultlocale()
  if default_lang:
    lang = default_lang
  if lang:
    os.environ['LANG'] = lang
import gettext
gettext.install("dsgui", os.path.join(topdir, "locale"))

# import and check gtk
GTK_REQUIRE = "2.0"
try:
  import pygtk
  pygtk.require("2.0") 
except:
  print _("Required PyGTK 2.0 was not found - try installing python-gtk, pygtk or similar package.")
  sys.exit()
try:
  import gtk
except:
  print _("Python module 'gtk' was not found - try installing python-gtk, pygtk or similar package.")
  sys.exit()


if __name__ == "__main__":
  GTK_REQUIRED_MINIMUM = (2,14,0)
  check = gtk.check_version(*GTK_REQUIRED_MINIMUM)
  if check: # check did not pass
    current_version = gtk.gtk_version
    message = _("""Your version of GTK is older that the required minimum.
  The application will continue, but it will probably not work properly.
  
  Information about versions:
    Your version: %d.%d.%d
    Required version: %d.%d.%d""") % (current_version+GTK_REQUIRED_MINIMUM)
    d = gtk.MessageDialog(None, message_format=message, buttons=gtk.BUTTONS_OK, type=gtk.MESSAGE_WARNING)
    d.run()
    d.destroy()
  del check
  
try:
  import gnomevfs
  GNOMEVFS_AVAILABLE = True
except:
  GNOMEVFS_AVAILABLE = False
import pango

from dslib.client import Client
from ds_search import DsSearchDialog
from ds_send import DsSendDialog
from proxy_dialog import ProxyDialog
from xml_translator import translate_xml_string
from icon_manager import IconManager
import gobject
import threading
from account import Credentials, Account


class GUI(object):

  topdir = topdir
  date_time_format = "%Y-%m-%d %H:%M"
  account_info_attrs = (("dbID",_("Data box ID")),("dbType",_("Data box type")),
                      ("ic",_("IČ")),("pnFirstName",_("Given name")),
                      ("pnMiddleName",_("Middle name")),("pnLastName",_("Surname")),
                      ("pnLastNameAtBirth",_("Surname at birth")),("firmName",_("Firm name")),
                      ("biDate",_("Date of birth")),("biCity",_("City of birth")),
                      ("biCounty",_("County of birth")),("biState",_("State of birth")),
                      ("adCity",_("City of residence")),("adStreet",_("Street of residence")),
                      ("adNumberInStreet",_("Number in street")),("adNumberInMunicipality",_("Number in municipality")),
                      ("adZipCode",_("Zip code")),("adState",_("State of residence")),
                      ("nationality",_("Nationality")),("dbEffectiveOVM",_("Effective OVM")),
                      ("dbOpenAddressing",_("Open addressing")),
                      )
  trusted_certificate_dir = os.path.join(topdir, "dslib", "trusted_certificates")
  progressbar = None
  gui_lock = threading.Lock()

  def __init__(self):
    self.proxy = -1
    self.received_messages = {} # id2message
    self.sent_messages = {}
    self.message_data = {} # id to data
    self.accounts = []
    self.account = None
    # icon manager
    IconManager.read_directory(os.path.join(self.topdir, "icons"))
    # build the GUI from xml
    builder = gtk.Builder()
    #builder.set_translation_domain("dsgui")
    self.builder = builder
    # the following adds some custom icons
    self._add_custom_icons()
    # // end of custom icon support
    gui_xml = GUI.fix_ui_xml("dsgui.ui")
    if gtk.gtk_version < (2,16,0):
      from ui_xml_versions import remove_incompatible_parts
      gui_xml_optimized = remove_incompatible_parts(gui_xml, gtk.gtk_version)
    else:
      gui_xml_optimized = gui_xml
    try:
      builder.add_from_string(gui_xml_optimized.decode('utf-8'))
    except:
      # this fixes problem with different API on windows and PyGTK 2.12
      builder.add_from_string(gui_xml_optimized.decode('utf-8'), -1)
    self.window1 = builder.get_object("window1")
    self._post_builder_ui_compatibility_fixes(gui_xml)
    builder.connect_signals(self)
    self.window1.show()
    GUI.progressbar = builder.get_object("main_progressbar")
    tag_table = builder.get_object("message_tagtable")
    self.received_message_store = builder.get_object("received_message_store")
    self.received_message_treeview = builder.get_object("received_message_treeview")
    self.received_message_textview = builder.get_object("received_message_textview")
    self.received_attachment_treeview = builder.get_object("received_attachment_treeview")
    self.received_attachment_store = builder.get_object("received_attachment_store")
    self.sent_message_store = builder.get_object("sent_message_store")
    self.sent_message_treeview = builder.get_object("sent_message_treeview")
    self.sent_message_textview = builder.get_object("sent_message_textview")
    self.account_store = builder.get_object("account_store")
    self.account_treeview = builder.get_object("account_treeview")
    self.statusbar = builder.get_object("statusbar")
    self._download_context = self.statusbar.get_context_id("download")
    # for some reason, "message_tagtable" is associated only with the "sent" buffer,
    # not with "received" - I fix it here by hand
    # UPDATE - text tags don't work well with glade anyway - lets do it all by hand
    for tv_id in ("sent_message_textview","received_message_textview",
                  "account_info_textview","show_message_dialog_textview"):
      tb = self.builder.get_object(tv_id).get_buffer()
      try:
        tb.create_tag("bold", weight=pango.WEIGHT_BOLD)
      except:
        pass
      try:
        tb.create_tag("larger", scale=1.25, pixels_above_lines=10)
      except:
        pass
    # some tweaks to permit nice display of read messages
    col = builder.get_object("received_message_read_column")
    cell = builder.get_object("received_message_read_cell_renderer")
    col.set_cell_data_func(cell, self._render_read_icon)
    # the same for message verification
    col = builder.get_object("received_message_verified_column")
    cell = builder.get_object("received_message_verified_cell_renderer")
    col.set_cell_data_func(cell, self._render_verified_icon)
    # the same for message verification of sent messages
    col = builder.get_object("sent_message_verified_column")
    cell = builder.get_object("sent_message_verified_cell_renderer")
    col.set_cell_data_func(cell, self._render_verified_icon, 1)
    # bind some special features to username rendering
    col = builder.get_object("account_treeview_username_column")
    cell = builder.get_object("account_treeview_username_cellrenderer")
    col.set_cell_data_func(cell, self._render_account_name)
    # add sorting info by hand - it does not load properly from XML
    # in older version of gtk (2.16 and below)
    for colname, i in (("treeviewcolumn_id", 0),("treeviewcolumn_title",1),
                       ("treeviewcolumn_sender",2), ("treeviewcolumn7",3),
                       ("treeviewcolumn8",4), ("received_message_read_column",5),
                       ("treeviewcolumn3",0), ("treeviewcolumn4",1),
                       ("treeviewcolumn5",2), ("treeviewcolumn6",6),
                       ("treeviewcolumn9",4), ("treeviewcolumn10",5),
                       ):
      col = builder.get_object(colname)
      col.set_sort_column_id(i)
    # bind text boldness to read locally status for received_attachment_treeview
    for colname in ("treeviewcolumn_id","treeviewcolumn_title","treeviewcolumn_sender",
                    "treeviewcolumn7","treeviewcolumn8"):
      col = builder.get_object(colname)
      renderer = col.get_cell_renderers()[0]
      col.set_cell_data_func(renderer, self._render_read_text)
    # check if python version works with HTTPS proxy
    python_version = tuple(sys.version_info[0:3])
    if python_version >= (2,6,3):
      self.proxy_support_ok = True
    else:
      self.proxy_support_ok = False
    if not self.proxy_support_ok:
      action = self.builder.get_object("change_proxy_settings")
      action.set_sensitive(False)
      try:
        # set_tooltip is not supported in older versions of PyGTK
        action.set_tooltip(_("Your version of Python does not support HTTPS proxy!"))
      except:
        pass
      self.proxy = None
      self.statusbar.push(-1, _("Your version of Python does not support HTTPS proxy!"))
    # activate tooltip display in statusbar for menu
    self._activate_tooltips_for_main_menu()
    # load a stored config file
    self._load_config_file()
    # the following is used for saving window position and size
    self._position = None
    self._size = None
    # some manual callbacks
    self.received_message_treeview.get_selection().connect("changed",
                          self.on_received_message_treeview_selection_changed,
                          self.received_message_treeview)
    self.sent_message_treeview.get_selection().connect("changed",
                          self.on_sent_message_treeview_selection_changed)
    self.account_treeview.get_selection().connect("changed",
                          self.on_account_treeview_selection_changed)
    self.received_attachment_treeview.get_selection().connect("changed",
                          self.on_received_attachment_treeview_selection_changed)
    # open an account
    if not self.account:
      self.on_add_account_activate()
    self._check_empty_message_stores()    
    #print time.time() - START_TIME
     
  # Callbacks
  
  def on_connect_action_activate(self, widget):
    received_count, sent_count = 0, 0
    # get received messages
    reply, error = self.try_client_method(
      self.client.GetListOfReceivedMessages, (),
      error_message=_("It was not possible to download the list of received messages.")
      )
    if not error:
      if reply.status.dmStatusCode == "0000":
        for m in reply.data:
          self.add_received_message(m)
          received_count += 1
      # get sent messages
      reply, error = self.try_client_method(
        self.client.GetListOfSentMessages, (),
        error_message=_("It was not possible to download the list of sent messages.")
        )
      if not error:
        if reply.status.dmStatusCode == "0000":
          for m in reply.data:
            self.add_sent_message(m)
            sent_count += 1
    self.statusbar.push(self._download_context,
                        _("%d received and %d sent messages found on server")%(received_count,sent_count))
    self._check_empty_message_stores()
    if not error:
      # check info about the account
      reply, error = self.try_client_method(
        self.client.GetOwnerInfoFromLogin,
        (),
        error_message=_("It was not possible to download info about the account."))
      if not error:
        info = reply.data
        check_exp_pass = False # should we check expiring password
        # check info about the account
        reply, error = self.try_client_method(
          self.client.GetPasswordInfo,
          (),
          error_message=_("It was not possible to download info about password expiration."))
        if not error:
          if self.account.password_expiration_date != reply.data:
            check_exp_pass = True
            self.account.password_expiration_date = reply.data
        self.set_account_info(info)
        if check_exp_pass:
          self.check_expired_password(self.account)


  def on_about_activate(self, widget):
    dial = self.builder.get_object("aboutdialog1")
    dial.run()
    dial.hide()

  def on_received_message_treeview_selection_changed(self, treeselection, treeview):
    message = self._get_selected_received_message()
    if message:
      self.show_received_message(message)
      self.activate_message_actions(message)
      # scroll to selection - it can disappear when sorting by read locally status
      # and reading something
      model, paths = self.received_message_treeview.get_selection().get_selected_rows()
      if len(paths) == 1:
        self.received_message_treeview.scroll_to_cell(paths[0])
    else:
      self.activate_message_actions(None)
      self.received_message_textview.get_buffer().set_text("")
    self.on_received_attachment_treeview_selection_changed(None)


  def on_window1_destroy(self, w):
    self._save_state()
    gtk.main_quit()
    
  def on_window1_delete_event(self, w, v):
    self._position = w.get_position()
    self._size = w.get_size()

  def on_download_complete_message_activate(self, w):
    self._try_download_complete_selected_message(self.client.MessageDownload)
      
  def on_download_complete_message_signed_activate(self, w, textview=None):
    self._try_download_complete_selected_message(self.client.SignedMessageDownload,
                                                 textview=textview)
      
  def _try_download_complete_selected_message(self, method, textview=None):
    message = self._get_selected_received_message()
    reply, error = self.try_client_method(
      method,
      (message.dmID,),
      error_message=_("It was not possible to download the message."),
      )
    if not error:
      message = reply.data
      self.add_received_message(message)
      self.show_received_message(message, textview=textview)
      self.on_received_attachment_treeview_selection_changed(None)
      
  def on_verify_sent_message_signature_activate(self, w, textview):
    message = self._get_selected_sent_message()
    reply, error = self.try_client_method(
      self.client.SignedSentMessageDownload,
      (message.dmID,),
      error_message=_("It was not possible to download the message."),
      )
    if not error:
      message = reply.data
      self.add_sent_message(message)
      self.show_sent_message(message, textview=textview)
    

  def on_received_attachment_treeview_row_activated(self, tree, path, column):
    self.builder.get_object("open_attachment").activate()

  def on_save_attachment_activate(self, w):
    message, attachment = self._get_selected_attachment()
    if attachment:
      d = gtk.FileChooserDialog(title=_("Select where to save"),
                                parent=self.window1,
                                action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                buttons=(gtk.STOCK_OK,gtk.RESPONSE_ACCEPT,
                                         gtk.STOCK_CANCEL,gtk.RESPONSE_REJECT)
                                )
      d.set_current_name(attachment._dmFileDescr)
      ok = d.run()
      if ok == gtk.RESPONSE_ACCEPT:
        fullname = d.get_filename()
        dir, name = os.path.split(fullname)
        attachment.save_file(dir, name)
      else:
        pass
      d.destroy()

  def on_open_attachment_activate(self, w):
    message, attachment = self._get_selected_attachment()
    done = False
    if os.name == 'nt':
      # WINDOWS
      temp_file = self._create_temp_file_for_dmfile(attachment)
      try:
        os.startfile(temp_file)
        done = True
      except WindowsError:
        done = False
    else:
      # we will use an external command via subprocess
      command = None
      if GNOMEVFS_AVAILABLE:
        # GNOME
        mime_type = gnomevfs.get_mime_type_for_data(attachment.get_decoded_content())
        mime_app = gnomevfs.mime_get_default_application(mime_type)
        if mime_app:
          command = mime_app[2]
      elif os.environ.get('KDE_FULL_SESSION') == 'true':
        # KDE
        command = 'kde-open'
      elif sys.platform == 'darwin':
        command = 'open'
      # did we find anything?
      if command:
        import subprocess
        try:
          temp_file = self._create_temp_file_for_dmfile(attachment)
          application = subprocess.Popen([command,temp_file])
        except:
          pass
        else:
          done = True
      
    if not done:
      d = gtk.MessageDialog(self.window1,
                            message_format=_("I could not find an application to open this attachment. You can save it and open it manually."),
                            buttons=gtk.BUTTONS_OK,
                            type=gtk.MESSAGE_ERROR)
      d.run()
      d.destroy()
      

  def on_sent_message_treeview_selection_changed(self, treeselection):
    message = self._get_selected_sent_message()
    self.activate_message_actions(message)
    if message:
      self.show_sent_message(message)
    else:
      self.sent_message_textview.get_buffer().set_text("")

    
  def on_edit_credentials_activate(self, w):
    d = self.builder.get_object("credentials_dialog")
    login_entry = self.builder.get_object("login_entry")
    login_entry.set_text(self.account and self.account.credentials.username or "")
    login_entry.set_sensitive(False)
    password_entry = self.builder.get_object("password_entry")
    password_entry.set_text(self.account and self.account.credentials.password or "")
    test_account_button = self.builder.get_object("test_environment_checkbutton")
    if self.account.credentials.test_account == None:
      # make test account the default 
      test_account_button.set_active(True)
    else:
      test_account_button.set_active(self.account.credentials.test_account)
    result = d.run()
    if result == 1:
      password = password_entry.get_text()
      test_account = test_account_button.get_active()
      for row in self.account_store:
        if row[0]==self.account.credentials.username and row[2]==self.account.credentials.test_account:
          row[1] = password
          row[2] = test_account
          break
      self.account.credentials.test_account = test_account
      self.account.credentials.change_password(password)
      self.client = GUI._create_ds_client(self.account.credentials, proxy=self.proxy)
    else:
      pass # cancel or something like this was pressed
    d.hide() # hide, don't destroy - the dialog is created by Builder


  def on_notebook1_switch_page(self, w, page, page_num):
    if page_num != 0:
      self.on_sent_message_treeview_selection_changed(None)
    else:
      self.on_received_message_treeview_selection_changed(None, None)


  def on_find_data_box_activate(self, w):
    d = DsSearchDialog(self, self.client)
    result = d.run()
    d.hide()
    if result == 1:
      boxes = d.get_selected_box_names()
      m = "\n".join(["%s: %s"%(box[0],box[1]) for box in boxes])
      d = gtk.MessageDialog(self.window1,
                            message_format=_("You selected the following boxes:\n\n")+m,
                            buttons=gtk.BUTTONS_OK,
                            type=gtk.MESSAGE_INFO)
      d.run()
      d.destroy()
    
  def on_send_message_activate(self, w):
    dial = DsSendDialog(self, self.client)
    ok = False
    new = True
    while not ok:
      if not new:
        dial.show_all()
      new = False
      result = dial.run()
      dial.hide()
      if result == 1:
        # dummy operation at first to get a cookie
        reply, error = self.try_client_method(self.client.DummyOperation, (), handle_status=False)
        if not error:
          # now send the file
          error_message = _("It was not possible to send the message")
          envelope, dmfiles = dial.get_prepared_message()
          reply, error = self.try_client_method(self.client.CreateMessage,
                                                 (envelope, dmfiles),
                                                 error_message=error_message,
                                                 handle_status=False)
          message = reply.status.dmStatusMessage
          code = reply.status.dmStatusCode
          if code != "0000":
            mtype = gtk.MESSAGE_ERROR
            message_text = _("An error occurred when sending the message:")
          else:
            mtype = gtk.MESSAGE_INFO
            message_text = _("Message was successfully sent:")
            ok = True
          message_text += "\n\n"
          message_text += _("Reply status: %(message)s\nReply code: %(code)s")%{"message":message,"code":code}
          d = gtk.MessageDialog(self.window1,
                                message_format=message_text,
                                buttons=gtk.BUTTONS_OK,
                                type=mtype)
          d.run()
          d.destroy()
      else:
        break

  def on_received_message_treeview_row_activated(self, tree, path, column):
    read_col = self.builder.get_object("received_message_read_column")
    if column == read_col:
      model = tree.get_model() 
      iter = model.get_iter(path) 
      msgid = model.get_value(iter, 0)
      read = self.message_data[msgid]['read_locally']
      read = not read
      self.message_data[msgid]['read_locally'] = read
      model.set_value(iter, 5, read)
    else:
      # open the message in a new window
      dial = self.builder.get_object("show_message_dialog")
      message = self._get_selected_received_message()
      textview = self.builder.get_object("show_message_dialog_textview")
      self.show_received_message(message, textview=textview)
      dial.show_all()
      dial.run()
      dial.hide()
      self.received_message_treeview.emit("cursor-changed")

  def on_sent_message_treeview_row_activated(self, tree, path, column):
    # open the message in a new window
    dial = self.builder.get_object("show_message_dialog")
    message = self._get_selected_sent_message()
    textview = self.builder.get_object("show_message_dialog_textview")
    self.show_sent_message(message, textview=textview)
    dial.show_all()
    dial.run()
    dial.hide()
    self.sent_message_treeview.emit("cursor-changed")

  def _something_with_attachment_happened(self):
    message, attachment = self._get_selected_attachment()
    if attachment:
      self.builder.get_object("save_attachment").set_sensitive(True)
      self.builder.get_object("open_attachment").set_sensitive(True)
    else:
      self.builder.get_object("save_attachment").set_sensitive(False)
      self.builder.get_object("open_attachment").set_sensitive(False)
    
      
  def on_received_attachment_store_row_number_changed(self, model, path, iter=None):
    self._something_with_attachment_happened()

  def on_received_attachment_treeview_selection_changed(self, treeselection):
    self._something_with_attachment_happened()

  def on_account_treeview_selection_changed(self, treeselection):
    account_row = self._get_selected_account_row()
    account = self._get_account_object_for_account_row(account_row)
    if account:
      self._open_account(account)
    else:
      raise Exception("unexisting account was selected - this should not happen")
      
  def on_add_account_activate(self, w=None):
    d = self.builder.get_object("credentials_dialog")
    login_entry = self.builder.get_object("login_entry")
    login_entry.set_text("")
    login_entry.set_sensitive(True)
    password_entry = self.builder.get_object("password_entry")
    password_entry.set_text("")
    test_account_button = self.builder.get_object("test_environment_checkbutton")
    test_account_button.set_active(True)
    result = d.run()
    if result == 1:
      username = login_entry.get_text()
      password = password_entry.get_text()
      test_account = test_account_button.get_active()
      row_id = self.add_account(username, password, test_account)
      if row_id >= 0:
        self.account_treeview.set_cursor(row_id)
      else:
        message = _("%(account_type)s '%(username)s' is already present, you cannot add it twice!")
        account_type = self._get_account_type_as_text(test_account)
        message = message % locals()
        dial = gtk.MessageDialog(self.window1,
                                 message_format=message,
                                 buttons=gtk.BUTTONS_OK,
                                 type=gtk.MESSAGE_ERROR)
        dial.run()
        dial.destroy()
      self._check_zero_accounts()
    else:
      # cancel or something like this was pressed
      self._check_zero_accounts()
    d.hide() # hide, don't destroy - the dialog is created by Builder

  def on_account_treeview_key_release_event(self, w, event):
    if event.keyval == gtk.keysyms.Delete:
      account = self._get_selected_account_row()
      if account:
        self.builder.get_object("remove_account").activate()

  def on_show_help_index_activate(self, w=None):
    import webbrowser
    webbrowser.open(GUI.topdir+"/doc/index.html")
    url = "file://"+GUI.topdir+"/doc/index.html"
    d = gtk.MessageDialog(self.window1,
                          message_format=_("Help index should open in your browser.\nIf it does not, you can use the following URL:\n%s")%url,
                          buttons=gtk.BUTTONS_OK,
                          type=gtk.MESSAGE_INFO)
    d.run()
    d.destroy()
    
  def on_change_proxy_settings_activate(self, w):
    dial = ProxyDialog(self, self.client)
    result = dial.run()
    if result == 1:
      proxy = dial.get_proxy()
      self.set_proxy(proxy)
    dial.destroy()

  def on_change_password_activate(self, w):
    from change_password import DsChangePasswordDialog
    d = DsChangePasswordDialog(self, self.client, self.account.account_info.dbID)
    new = True
    go_on = True
    while go_on:
      if not new:
        d.show_all()
      new = False
      result = d.run()
      d.hide()
      if result:
        old, new = d.get_passwords()
        reply, error = self.try_client_method(
          self.client.ChangeISDSPassword, (old, new),
          error_message=_("It was not possible to change the password.")
        )
        if not error:
          d.destroy()
          go_on = False
          # we must update the password we use for login as well
          for row in self.account_store:
            if row[0]==self.account.credentials.username and row[2]==self.account.credentials.test_account:
              row[1] = new
          self.account.credentials.change_password(new)
          self.client = GUI._create_ds_client(self.account.credentials, proxy=self.proxy)
          subd = gtk.MessageDialog(self.window1,
                                   message_format=_("Password was successfully changed."),
                                   buttons=gtk.BUTTONS_OK,
                                   type=gtk.MESSAGE_INFO)
          subd.run()
          subd.destroy()
          # update info about password expiration
          reply, error = self.try_client_method(
             self.client.GetPasswordInfo,
             (),
             error_message=_("It was not possible to download info about password expiration."))
          if not error:
            self.account.password_expiration_date = reply.data
            self._show_account_info()
        else:
          go_on = True
      else:
        go_on = False
  
  def on_remove_account_activate(self, w):
    account = self._get_selected_account_row()
    if account:
      d = gtk.MessageDialog(self.window1,
                            message_format=_("Remove account '%s'") % account[0],
                            buttons=gtk.BUTTONS_OK_CANCEL,
                            type=gtk.MESSAGE_QUESTION)
      answer = d.run()
      if answer == gtk.RESPONSE_OK:
        treeview = self.account_treeview
        model, paths = treeview.get_selection().get_selected_rows()
        self.account_store.remove(self.account_store.get_iter(paths[0]))
      d.destroy()

  def on_show_signature_details_activate(self, w, message=None):
    if not message:
      message, _ = self._get_selected_message_and_type()
    if message:
      from cert_display import CertDisplayDialog
      d = CertDisplayDialog(self.window1,
                            message)
      d.run()
      d.destroy()


  # // end of Callbacks

  # helper methods for display

  def _render_read_icon(self, column, cell, model, iter):
    data = model.get_value(iter, 5)
    if data == True:
      stock = gtk.STOCK_YES
    else:
      stock = gtk.STOCK_NO
    pixbuf = self.window1.render_icon(stock_id=stock, size=gtk.ICON_SIZE_MENU)
    cell.set_property('pixbuf', pixbuf)
    
  def _render_verified_icon(self, column, cell, model, iter, shift=0):
    ver = model.get_value(iter, 6+shift)
    sig = model.get_value(iter, 7+shift)
    attempt = model.get_value(iter, 8+shift)
    if not attempt:
      color = "grey"
    elif ver:
      if sig:
        color = "green"
      else:
        color = "orange"
    else:
      color = "red"
    #pixbuf = self.window1.render_icon(stock_id=stock, size=gtk.ICON_SIZE_MENU)
    cell.set_property('pixbuf', IconManager.get_pixbuf_by_name("key_"+color))   
  
  def _render_account_name(self, column, cell, model, iter):
    data = model.get_value(iter, 2)
    if data == True:
      style = pango.STYLE_ITALIC
    else:
      style = pango.STYLE_NORMAL
    cell.set_property('style', style)

  def _render_read_text(self, column, cell, model, iter):
    data = model.get_value(iter, 5)
    if data == True:
      cell.set_property('weight', pango.WEIGHT_NORMAL)
    else: 
      cell.set_property('weight', pango.WEIGHT_BOLD)


  def _fill_message_data_into_textview(self, message, textview, additional_lines=None):
    def add_row(heading, text, add_tags=None, newline=True):
      add = add_tags or ()
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, heading, "bold", *add)
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, " %s"%text, *add)
      if newline:
        add_newline()
    def add_newline():
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, "\n")  

    # fill the text buffer
    textbuffer = textview.get_buffer()
    textbuffer.set_text("")
    add_row(_("Subject:"), message.dmAnnotation, add_tags=("larger",))
    for name, value in (("dmLegalTitleLaw",_("Law")),
                        ("dmLegalTitleYear",_("Year")),
                        ("dmLegalTitleSect",_("Section")),
                        ("dmLegalTitlePar",_("Paragraph")),
                        ("dmLegalTitlePoint",_("Letter"))):
      if getattr(message, name):
        add_row(value+":", getattr(message, name))
    add_newline()
    add_row(_("From:"), message.dmSender)
    add_row(_("Sender Address:"), message.dmSenderAddress)
    add_newline()
    add_row(_("To:"), message.dmRecipient)
    add_row(_("Recipient Address:"), message.dmRecipientAddress)
    add_newline()
    add_row(_("Delivery time:"), self._format_datetime(message.dmDeliveryTime))
    add_row(_("Acceptance time:"), self._format_datetime(message.dmAcceptanceTime))
    add_newline()
    add_row(_("Status:"), "%d - %s" % (message.dmMessageStatus, message.get_status_description()))
    add_newline()
    if additional_lines:
      for head, text in additional_lines:
        add_row(head, text)
    # events - we would need to get delivery info for this, leave it for now
    #add_newline()
    #add_row("", "---------------------------------------")
    #for event in message.dmEvents:
    #  add_row(event.dmEventTime, event.dmEventDescr)
    # signatures - once download of signed messages works, info will go here
    # separator
      add_newline()
    if not message.message_verification_attempted():
      verified = _("Not present")
      add_row(_("Message signature:"), verified)
      position = textbuffer.get_end_iter()
      anchor = textbuffer.create_child_anchor(position)
      btn = gtk.Button(_("Verify now!"))
      textview.add_child_at_anchor(btn, anchor)
      if int(message.dmID) in self.received_messages:
        btn.connect("clicked", self.on_download_complete_message_signed_activate, textview)
      elif int(message.dmID) in self.sent_messages:
        btn.connect("clicked", self.on_verify_sent_message_signature_activate, textview)
      btn.show()
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, "\n")
    else:
      warning = _("Message signature and content do not correspond!")
      verified = message.is_message_verified() and _("Valid") or _("Invalid")+" - "+warning
      add_row(_("Message signature:"), verified)
      if message.is_message_verified():
        verified = message.is_signature_verified() and _("Valid") or _("Invalid")
        add_row(_("Signing certificate:"), verified)
    tst_check = message.check_timestamp()
    if tst_check == None:
      tst_text = _("Not present")
    else:
      timestamp_check = tst_check and _("Valid") or _("Invalid!")
      if message.tstamp_token:
        timestamp_time = message.tstamp_token.get_genTime_as_datetime()
        tst_text = "%s (%s)" % (timestamp_check, GUI._format_datetime(timestamp_time))
      else:
        tst_text = timestamp_check  
    add_row(_("Timestamp:"), tst_text)
    # add button for displaying of details about the signature
    position = textbuffer.get_end_iter()
    anchor = textbuffer.create_child_anchor(position)
    btn = gtk.Button(_("Signature details"))
    textview.add_child_at_anchor(btn, anchor)
    btn.connect("clicked", self.on_show_signature_details_activate, message)
    btn.show()
    #
    

  # // end of helper methods for display

  def activate_message_actions(self, message):
    message_actions = ["download_complete_message","download_complete_message_signed",
                     "save_attachment","open_attachment","show_signature_details"]
    if not message:
      for action in message_actions:
        self.builder.get_object(action).set_sensitive(False)
    else:
      if int(message.dmID) in self.sent_messages:
        # sent message
        for action in message_actions:
          if action == "show_signature_details":
            self.builder.get_object(action).set_sensitive(True)
          else:
            self.builder.get_object(action).set_sensitive(False)
      else:
        # received message
        received = True
        for action in message_actions:
          if action == "download_complete_message":
            if hasattr(message, 'dmFiles') and message.dmFiles:
              self.builder.get_object(action).set_sensitive(False)
            else:
              self.builder.get_object(action).set_sensitive(True)
          elif action in ("save_attachment","open_attachment"):
            self.builder.get_object(action).set_sensitive(False)
          else:
            self.builder.get_object(action).set_sensitive(True)

  def show_received_message(self, message, textview=None):
    if message:
      if not textview:
        textview = self.received_message_textview
      attachment_store = self.received_attachment_store
      if hasattr(message, 'dmFiles') and message.dmFiles:
        attachments = (_("Attachments:"), _("%d (downloaded and ready)")%len(message.dmFiles))
      else:
        attachments = (_("Attachments:"), _("not downloaded yet, ~%s KB; use 'Download' to get them.")%message.dmAttachmentSize)
      self._fill_message_data_into_textview(message, textview, additional_lines=[attachments])
      # show the attachments
      attachment_store.clear()
      for i,f in enumerate(message.dmFiles):
        attachment_store.append([i,f._dmFileDescr,f.get_size(),f._dmMimeType,f._dmFormat])
      # mark message read
      mid = int(message.dmID)
      if mid in self.message_data:
        if not self.message_data[mid]['read_locally']:
          self.message_data[mid]['read_locally'] = True
          for row in self.received_message_store:
            if row[0] == mid:
              row[5] = True
              break


  def show_sent_message(self, message, textview=None):
    if not textview:
      textview = self.sent_message_textview
    if message:
      self._fill_message_data_into_textview(message, textview)


  def add_received_message(self, message):
    msgid = int(message.dmID)
    if msgid not in self.message_data:
      self.message_data[msgid] = {'read_locally': False}
    if msgid in self.received_messages:
      if message.get_origin() != 'dmReturnedMessage' and \
        self.received_messages[msgid].get_origin() == "dmReturnedMessage":
        # this is a complete message - do not replace it
        # (it could be useful to create a list of increasing origin importance)
        pass
      else:
        attempt = message.message_verification_attempted()
        ver = message.is_message_verified()
        sig = message.is_signature_verified()
        for row in self.received_message_store:
          if row[0] == int(message.dmID):
            row[6] = ver
            row[7] = sig
            row[8] = attempt
            break
        self.received_messages[msgid] = message
    else:
      self.received_messages[msgid] = message
      annot = message.dmAnnotation
      sender = message.dmSender
      delivery_time = self._format_datetime(message.dmDeliveryTime)
      accept_time = self._format_datetime(message.dmAcceptanceTime)
      read = self.message_data[msgid]['read_locally']
      ver = message.is_message_verified()
      sig = message.is_signature_verified()
      attempt = message.message_verification_attempted()
      data = [msgid,annot,sender,delivery_time,accept_time,read,ver,sig,attempt]
      self.received_message_store.append(data)


  def add_sent_message(self, message):
    msgid = int(message.dmID)
    if msgid in self.sent_messages:
      if message.get_origin() != 'dmReturnedMessage' and \
        self.sent_messages[msgid].get_origin() == "dmReturnedMessage":
        # do not update - the original data is more specific
        return
    msgid = int(message.dmID)
    self.sent_messages[msgid] = message
    delivery_time = self._format_datetime(message.dmDeliveryTime)
    accept_time = self._format_datetime(message.dmAcceptanceTime)
    status = message.dmMessageStatus
    status_str = message.get_status_description()
    ver = message.is_message_verified()
    sig = message.is_signature_verified()
    attempt = message.message_verification_attempted()
    data = [msgid,message.dmAnnotation,message.dmRecipient,
            status_str,delivery_time,accept_time,status,ver,sig,attempt]
    for i,row in enumerate(self.sent_message_store):
      if row[0] == msgid:
        self.sent_message_store[i] = data
        break
    else:
      self.sent_message_store.append(data)
      
  def add_account(self, username, password, test_account):
    for row in self.account_store:
      if row[0] == username and row[2] == test_account:
        # this account is already present - cannot add it twice
        return -1
    # try to add into to the tooltip of an account
    account = Account(Credentials(username,password,test_account=test_account))
    text = self._get_tooltip_for_account(account)
    iter = self.account_store.append([username,password,test_account,text])
    self.accounts.append(account)
    return self.account_store.get_path(iter)

  def set_account_info(self, info):
    self.account.account_info = info
    self._show_account_info()
    # update info in tooltip
    account = self._get_selected_account_row()
    if account:
      account[3] = self._get_tooltip_for_account(self.account)
      
  def set_proxy(self, proxy):
    if proxy != self.proxy:
      self.proxy = proxy
      # update client if proxy changed
      self.client = self._create_ds_client(self.account.credentials, proxy=self.proxy)  
      
  # obsolete - removal candidate
  def check_expired_passwords(self):
    for account in self.accounts:
      self.check_expired_password(account)

  def check_expired_password(self, account):
    if account.has_password_expired():
      inp = {"username": account.credentials.username}
      message = _("""According to the last available information, your password for login '%(username)s' has expired.

Unless you have already done so, you need to use the official web interface of Datové schránky to change it.""") % inp
    elif account.is_password_expiring(10):
      delta = account.get_password_expiration_delta()
      inp = {"days": delta.days,
             "date": account.password_expiration_date,
             "username": account.credentials.username} #.strftime(locale.nl_langinfo(locale.D_T_FMT)).strip()}
      message = _("""According to the last available information, your password for login '%(username)s' will expire in %(days)s days (%(date)s).

You can change your password using the 'Databox/Change password' command.
Your new password will be valid for 90 days.""" % inp)
    else:
      return
    d = gtk.MessageDialog(self.window1,
                          message_format=message,
                          buttons=gtk.BUTTONS_OK,
                          type=gtk.MESSAGE_WARNING)
    d.run()
    d.destroy()   

  def _get_tooltip_for_account(self, account):
    info = account.account_info
    text = ""
    if info:
      for attr,name in GUI.account_info_attrs:
        if attr in ('dbID','pnFirstName','pnLastName',"biDate","firmName"):
          value = getattr(info, attr)
          if value:
            text += "<b>%s</b>: %s\n" % (name, value)
    text += "<b>%s</b>: %s" % (_("Account type"), self._get_account_type_as_text(account.credentials.test_account))
    return text

  def _show_account_info(self):
    # add info to the account_textview
    textbuffer = self.builder.get_object("account_info_textbuffer")
    textbuffer.set_text("")
    textbuffer.insert_with_tags_by_name(
      textbuffer.get_end_iter(),
      "%s\n" % self._get_account_type_as_text(self.account.credentials.test_account),
      'larger')
    textbuffer.insert_at_cursor("\n")
    info = self.account.account_info
    if info:
      for attr,name in GUI.account_info_attrs:
        if hasattr(info, attr):
          value = getattr(info, attr)
          if value:
            position = textbuffer.get_end_iter()
            textbuffer.insert_with_tags_by_name(position, name+":", "bold")
            position = textbuffer.get_end_iter()
            textbuffer.insert_at_cursor(" %s\n"%value)
      textbuffer.insert_at_cursor("\n")
      position = textbuffer.get_end_iter()
      textbuffer.insert_with_tags_by_name(position, _('Password expiration date:'), "bold")
      position = textbuffer.get_end_iter()
      textbuffer.insert_at_cursor(" %s\n"%(self.account.password_expiration_date or _("unknown")))
    else:
      position = textbuffer.get_end_iter()
      textbuffer.insert_at_cursor(_("Data has not been downloaded yet."))
   

  def _get_selected_message_and_type(self):
    notebook = self.builder.get_object("notebook1")
    if notebook.get_current_page() == 0:
      return self._get_selected_received_message(), "received"
    elif notebook.get_current_page() == 1:
      return self._get_selected_sent_message(), "sent"
    else:
      return None, None

  def _get_selected_received_message(self):
    treeview = self.received_message_treeview
    model, paths = treeview.get_selection().get_selected_rows()
    if len(paths) == 1:
      message = self.received_messages[model[paths[0]][0]]
      return message
    return None
    
  def _get_selected_attachment(self):
    message = self._get_selected_received_message()
    if message:
      treeview = self.received_attachment_treeview
      model, paths = treeview.get_selection().get_selected_rows()
      if len(paths) == 1:
        attachment = message.dmFiles[model[paths[0]][0]]
        return message, attachment
      return message, None
    return None, None

  def _get_selected_sent_message(self):
    treeview = self.sent_message_treeview
    model, paths = treeview.get_selection().get_selected_rows()
    if len(paths) == 1:
      message = self.sent_messages[model[paths[0]][0]]
      return message
    return None
      
      
  def _get_selected_account_row(self):
    treeview = self.account_treeview
    model, paths = treeview.get_selection().get_selected_rows()
    if len(paths) == 1:
      return model[paths[0]]
    return None
  
  def _get_account_object_for_account_row(self, row):
    for account in self.accounts:
      if account.credentials.username == row[0]:
        return account
    return None

  def _save_state(self):
    # dump messages
    shelf = self._open_shelf()
    self._store_to_shelf(shelf)
    shelf.close()
    # save config file
    import ConfigParser
    config = ConfigParser.RawConfigParser()
    # call individual saving methods
    self._save_credentials(config)
    self._save_gui_state(config)
    self._save_connection_state(config)
    # write it out and chmod to user-only access
    cr_filename = self._get_config_filename()
    cr_file = file(cr_filename, "wb")
    config.write(cr_file)
    cr_file.close()
    import stat
    os.chmod(cr_filename, stat.S_IREAD|stat.S_IWRITE)


  def _open_account(self, account):
    """called when the an account is selected from the account list"""
    if self.account and self.account == account:
      pass
    else:
      # we need to replace the content of the UI
      shelf = self._open_shelf()
      if self.account:
        # we have a username, so we save the current state
        self._store_to_shelf(shelf)
      self.received_messages = {}
      self.sent_messages = {}
      self.message_data = {}
      self.sent_message_store.clear()
      self.received_message_store.clear()
      self.received_attachment_store.clear()
      self.received_message_textview.get_buffer().set_text("")
      self.sent_message_textview.get_buffer().set_text("")
      self.builder.get_object("account_info_textbuffer").set_text("")
      self.builder.get_object("download_complete_message").set_sensitive(False)
      self.account = account
      self._load_from_shelf(shelf)
      shelf.close()
      self._check_zero_accounts()
      self._check_empty_message_stores()

  def _open_known_account(self, username):
    for i, row in enumerate(self.account_store):
      if row[0] == username:
        account = self._get_account_object_for_account_row(row)
        self._open_account(account)
        self.account_treeview.get_selection().select_path((i,))
        break

  def _open_shelf(self):
    import shelve
    fname = os.path.join(GUI.get_dsgui_private_dir(create=True), "messages.shelf")
    encoding = sys.getfilesystemencoding()
    if encoding:
      fname = fname.encode(encoding)
    shelf = shelve.open(fname, protocol=-1)
    return shelf


  def _store_to_shelf(self, shelf):
    if self.account:
      key = self._get_shelf_key()
      shelf[key] = {"received_messages":self.received_messages,
                    "sent_messages":self.sent_messages,
                    "message_data":self.message_data,
                    "account_info":self.account.account_info,
                    "password_expiration_date":self.account.password_expiration_date,
                    }
      shelf.sync()


  def _load_from_shelf(self, shelf):
    data = shelf.get(self._get_shelf_key(), None)
    check_exp_pass = False
    if data:
      message_data = data.get("message_data", None)
      if message_data:
        self.message_data = message_data
      for id,message in data['received_messages'].iteritems():
        self.add_received_message(message)
      for id,message in data['sent_messages'].iteritems():
        self.add_sent_message(message)
      account_info = data.get("account_info", None)
      ped = data.get("password_expiration_date", None)
      if ped != self.account.password_expiration_date:
        self.account.password_expiration_date = ped
        check_exp_pass = True
      self.set_account_info(account_info)
      if check_exp_pass:
        self.check_expired_password(self.account)
      

  def _get_shelf_key(self):
    """returns a key to be used for shelf based on current credentials"""
    assert self.account != None
    return GUI._create_shelf_key(self.account.credentials.username, self.account.credentials.test_account)


  def _show_error(self, message, exception=None):
    if exception:
      message += "\n\n"+_("Details:")+" "+str(exception)
      exc_type, exc_value, tb = sys.exc_info()
      import traceback
      tb_lines = traceback.format_tb(tb)
      print "\n".join(tb_lines)
      if len(tb_lines) > 5:
        # limit traceback to 20 lines
        tb_lines = ["..."]+tb_lines[-5:]
      traceback_text = "\n".join(tb_lines)
      #message += "\n\n"+traceback_text
    d = gtk.MessageDialog(self.window1,
                          message_format=message,
                          buttons=gtk.BUTTONS_OK,
                          type=gtk.MESSAGE_ERROR)
    d.run()
    d.destroy()

  def _handle_authorization_error(self, e, message_header=None):
    """convenience method to display information about failed authorization;
    returns True if the exception was handled, False otherwise"""
    if type(e.args) in (tuple,list) and \
    len(e.args) > 0 and type(e.args[0]) == tuple and len(e.args[0]) > 0:
      if e.args[0][0] == 401:
        # Unauthorized
        message = message_header and message_header+"\n\n" or ""
        message += _("""Authorization failed!

Please check your credentials including the test-environment setting.
It is possible that your password has expired - in this case,
you need to use the official web interface of Datové schránky to change it.""")
        self._show_error(message)
        return True
    return False
  
  def _save_credentials(self, config):
    """save current creadentials to a file"""
    for i, row in enumerate(self.account_store):
      num = ((i>0) and str(i+1) or "")
      section = "credentials"+num
      config.add_section(section)
      username, password, test_account = list(row)[0:3]
      config.set(section, 'username', username)
      config.set(section, 'password', password)
      config.set(section, 'test_account', test_account)
    current = self._get_selected_account_row()
    if current:
      config.add_section("default_account")
      config.set("default_account", "username", current[0])

  def _save_gui_state(self, config):
    config.add_section("message_ordering")
    for tree_name in ("received", "sent"):
      rec_col, rec_ord = getattr(self, tree_name+"_message_store").get_sort_column_id()
      if rec_col:
        config.set('message_ordering', tree_name+'_sort_column', rec_col)
        if rec_ord != None:
          rec_ord = rec_ord.value_name[4:]
          config.set('message_ordering', tree_name+'_sort_order', rec_ord)
    config.add_section("window_position")
    if self._position and self._size:
      x, y = self._position
      w, h = self._size
      config.set("window_position", "x", x)
      config.set("window_position", "y", y)
      config.set("window_position", "w", w)
      config.set("window_position", "h", h)

  def _save_connection_state(self, config):
    config.add_section("connection")
    config.set('connection', 'https_proxy', self.proxy)

  def _load_gui_state(self, config):
    for tree_name in ("received", "sent"):
      if config.has_option("message_ordering", tree_name+"_sort_column"):
        rec_col = config.getint("message_ordering", tree_name+"_sort_column")
        rec_ord = gtk.SORT_ASCENDING
        if config.has_option("message_ordering", tree_name+"_sort_order"):
          try:
            rec_ord = getattr(gtk, config.get("message_ordering", tree_name+"_sort_order"))
          except:
            pass
        getattr(self, tree_name+"_message_store").set_sort_column_id(rec_col, rec_ord)
    x, y, w, h = None, None, None, None
    if config.has_option("window_position", "x"):
      x = config.getint("window_position", "x")
    if config.has_option("window_position", "y"):
      y = config.getint("window_position", "y")
    if config.has_option("window_position", "w"):
      w = config.getint("window_position", "w")
    if config.has_option("window_position", "h"):
      h = config.getint("window_position", "h")
    if x is not None and y is not None and w is not None and h is not None:
      sw = self.window1.get_screen().get_width()
      sh = self.window1.get_screen().get_height()
      if x < sw and y < sh:
        self.window1.move(x, y)
      else:
        self.window1.move(0, 0)
      self.window1.resize(w, h)

  def _load_connection_state(self, config):
    if not self.proxy_support_ok:
      # this version of Python does not support HTTPS proxy anyway
      self.proxy = None
      return
    if config.has_section("connection"):
      if config.has_option('connection', 'https_proxy'):
        proxy = config.get('connection', 'https_proxy')
        if proxy == "-1":
          self.proxy = -1
        elif proxy == "None":
          self.proxy = None
        else:
          self.proxy = proxy
            
  def _load_config_file(self):
    cr_filename = self._get_config_filename()
    import ConfigParser
    config = ConfigParser.RawConfigParser()
    config.read(cr_filename)
    i = 1
    section = "credentials" + ((i>1) and str(i) or "")
    while config.has_section(section):
      if config.has_option(section, "username"):
        username = config.get(section, "username")
      else:
        username = ""
      if config.has_option(section, "password"):
        password = config.get(section, "password")
      else:
        password = ""
      if config.has_option(section, "test_account"):
        test_account = config.getboolean(section, "test_account")
      else:
        test_account = True
      self.add_account(username, password, test_account)
      i += 1
      section = "credentials" + ((i>1) and str(i) or "")
    self._load_gui_state(config)
    self._load_connection_state(config)
    if config.has_option("default_account", "username"):
      username = config.get("default_account", "username")
      self._open_known_account(username)
    elif self.accounts:
      self._open_account(self.accounts[0])
      self.account_treeview.get_selection().select_path((0,))


  def _get_config_filename(self):
    return os.path.join(GUI.get_dsgui_private_dir(create=True), "dsgui.conf")
  
  def _check_empty_message_stores(self):
    """check if the store of received or sent messages is empty
    and warn if need be.
    """
    rec_base = [0,_("No received messages"),"","","",True,False,False,False]
    sent_base = [0,_("No sent messages"),"","","","","",False,False,False]
    for treeview,message in ((self.received_message_treeview, rec_base),
                             (self.sent_message_treeview, sent_base)):
      model = treeview.get_model()
      for i,row in enumerate(model):
        if row[0] == 0:
          model.remove(model.get_iter(i))
          break
      if len(model) == 0:
        model.append(message)
        treeview.set_sensitive(False)
      else:
        treeview.set_sensitive(True)

  def _activate_tooltips_for_main_menu(self):
    def _X_(w):
      if w.get_action():
        tooltip = w.get_action().get_property("tooltip")
        if tooltip:
          self.statusbar.push(-1, tooltip)
    def _Y_(w):
      self.statusbar.pop(-1)
    def _act_one(x):
      for child in x:
        child.connect("select", _X_)
        child.connect("deselect", _Y_)
        submenu = child.get_submenu()
        if submenu:
          _act_one(submenu)
    menu = self.main_menu
    _act_one(menu)
    
  def _post_builder_ui_compatibility_fixes(self, gui_xml):
    if gtk.gtk_version < (2,16,0):
      # in 2.14 menu is not properly loaded from XML
      import xml.dom.minidom as dom
      doc = dom.parseString(gui_xml)
      from ui_xml_versions import manually_create_menu, bind_nonmenu_action_references
      menu = manually_create_menu(doc, "main_menu", self, self.builder)
      bind_nonmenu_action_references(doc, self.builder)
      old_menu = self.builder.get_object("main_menu")
      if old_menu:
        old_menu.destroy()
      self.builder.get_object("vbox1").pack_start(menu, False, False)
      self.builder.get_object("vbox1").reorder_child(menu, 0)
      menu.show_all()
    else:
      menu = self.builder.get_object("main_menu")
    self.main_menu = menu
    # now try hildon support (Maemo)
    try:
      import hildon
    except:
      pass
    else:
      self.app = hildon.Program()
      hildon_window = hildon.Window()
      self.app.add_window(hildon_window)
      self.builder.get_object("vbox1").reparent(hildon_window)
      hildon_window.set_title("dsgui")
      self.window1.destroy()
      new_menu = gtk.Menu()
      for child in menu.get_children():
        child.reparent(new_menu)
      hildon_window.set_menu(new_menu)
      menu.destroy()
      self.builder.get_object("toolbar1").destroy()
      self.builder.get_object("warning_label").destroy()
      hildon_window.connect("destroy", self.on_window1_destroy)
      self.window1 = hildon_window
    
  def try_client_method(self, method, args, error_message=None, handle_status=True, parent=None, async=True):
    """convenience function to run a method of self.client and display
    error messages, etc. as appropriate.
    Returns a tuple (reply, error_shown), where reply is the reply of the method and
    error_shown is a boolean telling if an error message was displayed.
    If parent is given then it is disabled while the soap thread is running instead of the main window
    """
    if not error_message:
      error_message = _("An error occured during the following operation: %s") % method.__name__
    # run the method in a separate thread
    parent = parent or self.window1
    exception = None
    reply = None
    if async:
      # asynchronous processing - we start a separate thread
      GUI.gui_lock.acquire()
      from soap_thread import SoapThread
      thread = SoapThread(method, args)
      self.progressbar.pulse()
      self.progressbar.set_text(method.__name__)
      parent.set_sensitive(False)
      thread.start()
      while thread.isAlive():
        gobject.idle_add(GUI.progressbar.pulse)
        time.sleep(0.1)
        while gtk.events_pending():
          gtk.main_iteration(block=False)
      self.progressbar.set_fraction(0.0)
      self.progressbar.set_text("Idle")
      parent.set_sensitive(True)
      GUI.gui_lock.release()
      exception = thread.exception
      reply = thread.reply
    else:
      # non-async - for some reason we do not want it
      try:
        reply = method(*args)
      except Exception, e:
        exception = e
      else:
        exception = None
    # see what we got back
    if exception:
      if not self._handle_authorization_error(exception, message_header=error_message):
        self._show_error(error_message, exception=exception)
      return None, True
    else:
      error_shown = False
      if handle_status:
        code, message = GUI._extract_code_and_message_from_reply(reply)
        if code != "0000":
          error_message += "\n\n"
          error_message += _("Reply status: %(message)s\nReply code: %(code)s")%{"message":message,"code":code}
          self._show_error(error_message)
          error_shown = True
      return reply, error_shown
    
  @classmethod
  def get_dsgui_private_dir(cls, create=False):
    homedir = cls.get_home_dir()
    priv_dir = os.path.join(homedir, ".dsgui")
    if create and not os.path.exists(priv_dir):
      os.mkdir(priv_dir)
    return priv_dir
  
  @classmethod
  def get_home_dir(cls):
    dir = None
    if os.name == "nt":
      try:
        import _winreg as reg
        dir = reg.QueryValueEx(reg.OpenKey(reg.HKEY_CURRENT_USER, "Volatile Environment"), "APPDATA")[0]
      except EnvironmentError:
        pass
    if not dir:
      dir = os.getenv('HOME') or cls.topdir
    return dir
    
  @classmethod
  def _get_account_type_as_text(self, test_account):
    account_type = test_account and _("Test account") or _("Standard account")
    return account_type
      
  @classmethod
  def _extract_code_and_message_from_reply(cls, reply):
    if not reply:
      return None, None
    if not hasattr(reply, 'status'):
      return None, None
    elif hasattr(reply.status, 'dmStatusCode'):
      return reply.status.dmStatusCode, reply.status.dmStatusMessage
    elif hasattr(reply.status, 'dbStatusCode'):
      return reply.status.dbStatusCode, reply.status.dbStatusMessage
    return None, None

  @classmethod
  def _create_ds_client(self, credentials, proxy=None):
    return Client(credentials.username, 
                  credentials.password, 
                  test_environment=credentials.test_account,
                  proxy=proxy,
                  trusted_certs_dir=self.trusted_certificate_dir)


  @classmethod
  def _create_temp_file_for_dmfile(self, dmfile):
    import tempfile
    ext = os.path.splitext(dmfile._dmFileDescr)[1]
    temp = tempfile.mkstemp(suffix=ext)
    os.write(temp[0], dmfile.get_decoded_content())
    os.close(temp[0])
    return temp[1]

  @classmethod
  def _format_datetime(cls, d):
    if d:
      return d.strftime(cls.date_time_format)
    return ""

  @classmethod
  def _create_shelf_key(cls, username, test_account):
    """returns a key to be used for shelf based on current credentials"""
    return "%s___%s" % (username, test_account)

  @classmethod
  def fix_ui_xml(cls, filename):
    """convenience function wrapping several one-purpose methods"""
    gui_xml = cls.compatibilize_ui_xml(filename).encode('utf-8')
    gui_xml = cls.translate_ui_xml(gui_xml)
    return gui_xml
  
  @classmethod
  def translate_ui_xml(cls, gui_xml):
    gui_xml = translate_xml_string(gui_xml, _)
    return gui_xml

  @classmethod
  def compatibilize_ui_xml(cls, filename):
    from ui_xml_versions import compatibilize_xml_file
    xml_string = compatibilize_xml_file(os.path.join(cls.topdir,filename), gtk.gtk_version)
    return xml_string

  @classmethod
  def _add_custom_icons(cls):
    # custom icon support
    f = gtk.IconFactory()
    # down with a key
    big = gtk.Image().render_icon(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_DIALOG)
    small = IconManager.get_pixbuf_by_name("stock_keyring")
    IconManager.add_composite("signed-download", big, small)
    iconset = gtk.IconSet(IconManager.get_pixbuf_by_name("signed-download"))
    f.add("signed-download", iconset)
    # unified ok (ok-state)
    iconset = gtk.IconSet(IconManager.get_pixbuf_by_name("ok"))
    f.add("ok-state", iconset)
    # unified error (error-state)
    iconset = gtk.IconSet(IconManager.get_pixbuf_by_name("error"))
    f.add("error-state", iconset)
    f.add_default()


  def _check_zero_accounts(self):
    actions = ("connect_action", "find_data_box", "send_message", "change_password","show_signature_details")
    if self.account:
        self.client = self._create_ds_client(self.account.credentials, proxy=self.proxy)
        for action in actions:
            self.builder.get_object(action).set_sensitive(True)
    else:
        self.client = None
        for action in actions:
            self.builder.get_object(action).set_sensitive(False)



if __name__ == "__main__":
  import logging
  logging.basicConfig(level=logging.CRITICAL)
  #logging.getLogger('pkcs7.rsa_verifier').setLevel(logging.CRITICAL)
  gobject.threads_init()
  app = GUI()
  gtk.main()